import styled from "styled-components";

export type TableColumn<T> = {
  label: string | JSX.Element;
  accessKey?: keyof T;
  renderCell?: ({ row, idx }: { row: T; idx: number }) => JSX.Element;
  renderValue?: ({ row, idx }: { row: T; idx: number }) => any;
  isVisible?: boolean;
  className?: string;
  width?: string;
};

export type TableProps<T> = {
  columns: TableColumn<T>[];
  tableData: T[];
  isLoading?: boolean;
};

export const Table = <T extends {}>({
  columns,
  tableData,
  isLoading = false,
}: TableProps<T>) => {
  const renderTbody = () => {
    if (tableData.length === 0) {
      return false;
    }

    return tableData.map((row, rowIndex) => (
      <tr key={rowIndex} className="table__tbody-row">
        {columns.map((column, columnIdx) => {
          if (column.isVisible === false) {
            return undefined;
          }

          const { accessKey, className, renderValue, renderCell } = column;

          let value = undefined;

          if (renderCell) {
            value = renderCell({ row, idx: rowIndex });
          } else if (renderValue) {
            value = renderValue({ row, idx: rowIndex });
          } else if (accessKey) {
            value = row[accessKey];
          }

          return (
            <td
              key={`${rowIndex}-${String(accessKey || columnIdx)}`}
              className={`table__cell ${className ? className : ""}`}
            >
              {value}
            </td>
          );
        })}
      </tr>
    ));
  };

  const renderSkeleton = () => {
    return Array.from({ length: 5 }, (_, rowIndex) => (
      <tr className="table__tbody-row table__tbody-row--skeleton">
        {columns.map((column, columnIdx) => {
          if (column.isVisible === false) {
            return undefined;
          }

          return (
            <td key={`${String(column.accessKey || columnIdx)}_${rowIndex}`}>
              <div className="skeleton-item" />
            </td>
          );
        })}
      </tr>
    ));
  };

  return (
    <Container id="common-table-container">
      <table className="table">
        <thead className="table__thead">
          <tr className="table__thead-row">
            {columns.map((column, columnIndex) => {
              if (column.isVisible === false) {
                return undefined;
              }

              return (
                <th
                  key={`${String(
                    column.accessKey || columnIndex
                  )}_${columnIndex}`}
                  className={`table__thead-cell ${
                    column.className ? column.className : ""
                  }`}
                  style={{ width: column.width }}
                >
                  {column.label}
                </th>
              );
            })}
          </tr>
        </thead>
        <tbody className="table__tbody">
          {isLoading ? renderSkeleton() : renderTbody()}
        </tbody>
      </table>

      {!isLoading && tableData.length === 0 && (
        <div className="table--empty">데이터가 없습니다.</div>
      )}
    </Container>
  );
};

const Container = styled.div`
  table {
    width: 100%;
    table-layout: fixed;
    border-collapse: collapse;

    thead th {
      padding: 0 8px;
      text-align: left;
    }

    tbody td {
      height: 40px;
      vertical-align: middle;
      padding: 0 8px;
    }

    .skeleton-item {
      width: 100%;
      height: 16px;
      background-color: lightgray;
      border-radius: 40px;
    }
  }

  .table--empty {
    height: 40px;
    line-height: 40px;
    text-align: center;
  }
`;
