import styled from "styled-components";
import { useCallback, useEffect, useState } from "react";
import Toaster, { ToastItem } from "./utils/toaster";
import Toast from "./components/Toast";

export type ToastPosition = "top" | "bottom";

export type ToastProviderProps = {
  /** 토스트 메시지가 화면에 표시되는 시간을 밀리초(ms) 단위 */
  duration?: number;
  /** 토스트 메시지 사이의 간격 지정 */
  gap?: number;
  /** 토스트가 화면의 맨 위 또는 맨 아래로부터 얼마나 떨어져 있는지를 설정 */
  offset?: number;
  /** 토스트 메시지가 표시될 위치를 지정 */
  position?: ToastPosition;
  zIndex?: number;
};

export let toast: Toaster = new Toaster(null);

export const ToastProvider = ({
  gap = 16,
  duration = 3200,
  offset = 0,
  position = "bottom",
  zIndex = 999,
}: ToastProviderProps) => {
  const [toasts, setToasts] = useState<ToastItem[]>([]);
  const [heights, setHeights] = useState<
    { height: number; id: ToastItem["id"] }[]
  >([]);

  const removeToast = useCallback((toastId: ToastItem["id"]) => {
    toast.removeToast(toastId);
    removeheightOfToast(toastId);
  }, []);

  const addHeightOfToast = useCallback(
    (toastId: ToastItem["id"], height: number) => {
      setHeights((state) => [{ id: toastId, height }, ...state]);
    },
    []
  );

  const removeheightOfToast = (toastId: ToastItem["id"]) => {
    setHeights((state) => state.filter(({ id }) => id !== toastId));
  };

  const calculateOffsetY = (idx: number) => {
    return heights
      .slice(0, idx)
      .reduce((acc, current) => acc + current.height + gap, 0);
  };

  useEffect(() => {
    toast = new Toaster(setToasts);
  }, []);

  return (
    <Container
      style={{
        transform: `translateY(${position === "top" ? offset : -offset}px)`,
        zIndex: zIndex,
      }}
    >
      {toasts.map((toast, idx) => (
        <Toast
          key={toast.id}
          toast={toast}
          duration={duration}
          position={position}
          offsetY={calculateOffsetY(idx)}
          addHeightOfToast={addHeightOfToast}
          removeToast={removeToast}
        />
      ))}
    </Container>
  );
};

const Container = styled.div`
  position: fixed;
  top: 0;
  right: 0;
  bottom: 0;
  left: 0;
  pointer-events: none;
`;
